package pl.edu.pg.eti.kask.rpg;

import pl.edu.pg.eti.kask.rpg.character.controller.CharacterSocketController;
import pl.edu.pg.eti.kask.rpg.character.initialize.InitializeTestData;
import pl.edu.pg.eti.kask.rpg.character.repository.CharacterRepository;
import pl.edu.pg.eti.kask.rpg.character.repository.ProfessionRepository;
import pl.edu.pg.eti.kask.rpg.character.repository.SkillRepository;
import pl.edu.pg.eti.kask.rpg.character.service.CharacterService;
import pl.edu.pg.eti.kask.rpg.character.service.ProfessionService;
import pl.edu.pg.eti.kask.rpg.character.service.SkillService;
import pl.edu.pg.eti.kask.rpg.character.view.DataList;
import pl.edu.pg.eti.kask.rpg.controller.SocketController;
import pl.edu.pg.eti.kask.rpg.controller.thread.IncomingSocketConnectionHandler;
import pl.edu.pg.eti.kask.rpg.export.service.ExportService;
import pl.edu.pg.eti.kask.rpg.export.service.ImportService;
import pl.edu.pg.eti.kask.rpg.export.view.ExportData;
import pl.edu.pg.eti.kask.rpg.export.view.ImportData;
import pl.edu.pg.eti.kask.rpg.session.service.SessionService;
import pl.edu.pg.eti.kask.rpg.session.view.SessionStart;

import java.nio.file.Path;
import java.util.List;
import java.util.Scanner;

/**
 * Application main class. Contains application main entry point.
 */
public class SimpleRpgApplication {

    /**
     * Application entry point.
     *
     * @param args command line arguments
     */
    public static void main(String[] args) {
        boolean initialize = args.length >= 1 && Boolean.parseBoolean(args[0]);

        SimpleRpgApplication application = new SimpleRpgApplication();

        ProfessionService professionService = application.createProfessionService();
        CharacterService characterService = application.createCharacterService();
        SkillService skillService = application.createSkillService();

        if (initialize) {
            InitializeTestData initializeTestData = new InitializeTestData(characterService, professionService, skillService);
            initializeTestData.init();
        }

        SessionService sessionService = new SessionService(characterService);

        ExportService exportService = new ExportService(skillService);
        ImportService importService = new ImportService(skillService);

        Scanner scanner = new Scanner(System.in);

        List<SocketController> controllers = List.of(new CharacterSocketController(characterService));
        Thread serverThread = new Thread(new IncomingSocketConnectionHandler(8080, controllers));
        serverThread.start();

        boolean running = true;

        while (running) {
            System.out.print("command: ");
            String command = scanner.next();
            switch (command) {
                case "list":
                    new DataList(characterService, professionService, skillService).display();
                    break;
                case "session":
                    new SessionStart(sessionService).display();
                    break;
                case "export":
                    new ExportData(exportService).display();
                    break;
                case "import":
                    new ImportData(importService).display();
                    break;
                case "quit":
                    running = false;
                    break;
                default:
                    System.out.println("export|import|session|list|quit");
            }
        }

        serverThread.interrupt();
    }

    /**
     * @return configures service
     */
    private ProfessionService createProfessionService() {
        return new ProfessionService(new ProfessionRepository(Path.of("professions.bin")));
    }

    /**
     * @return configures service
     */
    private CharacterService createCharacterService() {
        return new CharacterService(new CharacterRepository(Path.of("characters.bin")));
    }

    /**
     * @return configures service
     */
    private SkillService createSkillService() {
        return new SkillService(new SkillRepository(Path.of("skills.bin")));
    }

}
