package pl.edu.pg.eti.kask.rpg.controller.thread;

import lombok.extern.java.Log;
import pl.edu.pg.eti.kask.rpg.controller.SocketController;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.util.List;
import java.util.logging.Level;

/**
 * Central connection handler for request sent to the specified socket.
 */
@Log
public class IncomingSocketConnectionHandler implements Runnable {

    /**
     * Server socket used for fetching requests.
     */
    private final ServerSocket server;

    /**
     * Registered controllers.
     */
    private final List<SocketController> controllers;

    /**
     * @param port        number of port which will be used for listening
     * @param controllers registered controllers
     * @throws IllegalStateException on IO errors while creating a socket
     */
    public IncomingSocketConnectionHandler(int port, List<SocketController> controllers) {
        this.controllers = controllers;
        try {
            server = new ServerSocket(port);
            server.setSoTimeout(1000);
        } catch (IOException ex) {
            throw new IllegalStateException(ex);
        }
    }

    @Override
    public void run() {
        log.info("Staring HTTP listener.");
        while (!Thread.interrupted()) {
            try {
                Socket socket = server.accept();
                new Thread(new SocketConnectionHandler(socket, controllers)).start();
            } catch (SocketTimeoutException ex) {
                log.log(Level.FINEST, ex.getMessage(), ex);
            } catch (IOException ex) {
                log.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
        log.info("Stopping HTTP listener.");
    }

}
