package pl.edu.pg.eti.kask.rpg.export.service;

import pl.edu.pg.eti.kask.rpg.character.entity.Skill;
import pl.edu.pg.eti.kask.rpg.character.service.SkillService;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Service for exporting data as text files.
 */
public class ExportService {

    /**
     * Service for {@link pl.edu.pg.eti.kask.rpg.character.entity.Skill}.
     */
    private final SkillService skillService;

    /**
     * @param skillService service for {@link pl.edu.pg.eti.kask.rpg.character.entity.Skill}
     */
    public ExportService(SkillService skillService) {
        this.skillService = skillService;
    }

    /**
     * @param path file where all skills will be stored
     * @throws IllegalStateException on IO errors
     */
    public void exportAllSkills(Path path) throws IllegalStateException {
        try (BufferedWriter bw = Files.newBufferedWriter(path)) {
            for (Skill skill : skillService.findAllSkills()) {
                bw.write(skill.getName());
                bw.newLine();
                bw.write(skill.getDescription());
                bw.newLine();
            }
        } catch (IOException ex) {
            throw new IllegalStateException(ex);
        }
    }

}
