package pl.edu.pg.eti.kask.rpg.export.service;

import pl.edu.pg.eti.kask.rpg.character.entity.Skill;
import pl.edu.pg.eti.kask.rpg.character.service.SkillService;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Service for importing data from text files.
 */
public class ImportService {

    /**
     * Service for {@link Skill}.
     */
    private final SkillService skillService;

    /**
     * @param skillService service for {@link Skill}
     */
    public ImportService(SkillService skillService) {
        this.skillService = skillService;
    }

    /**
     * @param path file where all skills are stored
     * @throws IllegalStateException on IO errors
     */
    public void importAllSkills(Path path) throws IllegalStateException {
        try (BufferedReader br = Files.newBufferedReader(path)) {
            String name;
            String description;
            while ((name = br.readLine()) != null) {
                description = br.readLine();
                skillService.create(Skill.builder()
                        .name(name)
                        .description(description)
                        .build());
            }
        } catch (IOException ex) {
            throw new IllegalStateException(ex);
        }
    }

}
